// EtherRaw.cpp
//
// Copyright (C) 2012 Mike McCauley
// $Id: EtherRaw.cpp,v 1.1 2012/05/30 01:52:08 mikem Exp mikem $

#include "EtherRaw.h"
#include <util.h>

//
// IPv4Address
//

IPv4Address::IPv4Address()
{
    memset(_address, 0, sizeof(_address));
}

IPv4Address::IPv4Address(uint8_t octet1, uint8_t octet2, uint8_t octet3, uint8_t octet4)
{
    _address[0] = octet1;
    _address[1] = octet2;
    _address[2] = octet3;
    _address[3] = octet4;
}

IPv4Address::IPv4Address(const uint8_t *addr)
{
    memcpy(_address, &addr, sizeof(_address));
}

bool IPv4Address::operator==(const IPv4Address& addr) 
{ 
    return memcmp(addr._address, _address, sizeof(_address)) == 0;
}

bool IPv4Address::operator==(const uint8_t* addr)
{
    return memcmp(addr, _address, sizeof(_address)) == 0;
}

IPv4Address& IPv4Address::operator=(const uint8_t *addr)
{
    memcpy(_address, (const uint8_t *)&addr, sizeof(_address));
    return *this;
}

const uint8_t* IPv4Address::address() const
{ 
    return _address; 
}

//
// IPv6Address
//

IPv6Address::IPv6Address()
{
    memset(_address, 0, sizeof(_address));
}

IPv6Address::IPv6Address(const uint8_t *addr)
{
    memcpy(_address, &addr, sizeof(_address));
}

bool IPv6Address::operator==(const IPv6Address& addr) 
{ 
    return memcmp(addr._address, _address, sizeof(_address)) == 0;
}

bool IPv6Address::operator==(const uint8_t* addr)
{
    return memcmp(addr, _address, sizeof(_address)) == 0;
}

IPv6Address& IPv6Address::operator=(const uint8_t *addr)
{
    memcpy(_address, (const uint8_t *)&addr, sizeof(_address));
    return *this;
}

const uint8_t* IPv6Address::address() const
{ 
    return _address; 
}

//
// MACAddress
//
MACAddress::MACAddress()
{
    memset(_address, 0, sizeof(_address));
}

MACAddress::MACAddress(uint8_t octet1, uint8_t octet2, uint8_t octet3, uint8_t octet4, uint8_t octet5, uint8_t octet6)
{
    _address[0] = octet1;
    _address[1] = octet2;
    _address[2] = octet3;
    _address[3] = octet4;
    _address[4] = octet5;
    _address[5] = octet6;
}

MACAddress::MACAddress(const uint8_t *addr)
{
    memcpy(_address, &addr, sizeof(_address));
}

bool MACAddress::operator==(const MACAddress& addr) 
{ 
    return memcmp(addr._address, _address, sizeof(_address)) == 0;
}

bool MACAddress::operator==(const uint8_t* addr)
{
    return memcmp(addr, _address, sizeof(_address)) == 0;
}

uint8_t MACAddress::operator[](int index) const 
{ 
    return _address[index]; 
};

uint8_t& MACAddress::operator[](int index) 
{ 
    return _address[index]; 
};

MACAddress& MACAddress::operator=(const uint8_t *addr)
{
    memcpy(_address, (const uint8_t *)&addr, sizeof(_address));
    return *this;
}

uint8_t* MACAddress::address() 
{ 
    return _address; 
}

size_t  MACAddress::printTo(Print& p) const
{
    size_t n = 0;
    for (int i = 0; i < MAC_ADDRESS_LENGTH-1; i++)
    {
        n += p.print(_address[i], HEX);
        n += p.print(':');
    }
    n += p.print(_address[MAC_ADDRESS_LENGTH-1], HEX);
    return n;
}

//
// EthernetHeader
//

MACAddress& EthernetHeader::source() 
{ 
    return _source;
}

MACAddress& EthernetHeader::dest() 
{ 
    return _dest;
}

uint16_t  EthernetHeader::ethertype() const
{
    return ntohs(_ethertype);
}

uint8_t* EthernetHeader::payload()
{
    return _data; // REVISIT: account for 802.1Q header here
}

size_t  EthernetHeader::printTo(Print& p) const
{
    size_t n = 0;
    n += p.println("Ethernet");
    n += p.print(" From MAC: ");
    n += _source.printTo(p);
    n += p.println("");
    n += p.print(" To MAC: ");
    n += _dest.printTo(p);
    n += p.println("");
    n += p.print(" EtherType: ");
    n += p.println(ethertype(), HEX);
    return n;
}

//
// PacketIPv4
//

uint8_t PacketIPv4::version() const 
{
    return _verslen >> 4;
}

uint8_t PacketIPv4::headerlen() const 
{
    return (_verslen & 0xf) << 2;
}

uint8_t PacketIPv4::service() const 
{
    return _service & IPTOS_TOS_MASK;
}

uint8_t PacketIPv4::precedence() const 
{
    return _service >> 5;
}

uint16_t  PacketIPv4::length() const
{
    return ntohs(_length);
}

uint16_t  PacketIPv4::id() const
{
    return ntohs(_id);
}

uint8_t  PacketIPv4::flags() const
{
    return ntohs(_flags_offset) >> 13;
}

uint16_t  PacketIPv4::offset() const
{
    return ntohs(_flags_offset) & 0x1fff;
}

uint8_t PacketIPv4::ttl() const 
{
    return _ttl;
}

uint8_t PacketIPv4::protocol() const 
{
    return _protocol;
}

const IPv4Address& PacketIPv4::source() const 
{
    return _source;
}

const IPv4Address& PacketIPv4::dest() const 
{
    return _dest;
}

uint16_t  PacketIPv4::checksum() const
{
    return ntohs(_checksum);
}

uint8_t* PacketIPv4::payload()
{
    return (uint8_t*)this + headerlen();
}

size_t  PacketIPv4::printTo(Print& p) const
{
    size_t n = 0;
    n += p.println("IP");
    n += p.print(" Vers Len: ");
    n += p.println(_verslen, HEX);
    n += p.print(" Service: ");
    n += p.println(service(), HEX);
    n += p.print(" Length: ");
    n += p.println(length(), DEC);
    n += p.print(" Id: ");
    n += p.println(id(), DEC);
    n += p.print(" Flags/Offset: ");
    n += p.println(_flags_offset, HEX);
    n += p.print(" TTL: ");
    n += p.println(ttl(), DEC);
    n += p.print(" Protocol: ");
    n += p.println(protocol(), DEC);
    n += p.print(" Checksum: ");
    n += p.println(checksum(), HEX);

    IPAddress tmpsource(((IPv4Address&)_source).address());
    n += p.print(" Source: ");
    n += tmpsource.printTo(p);
    n += p.println("");

    IPAddress tmpdest(((IPv4Address&)_dest).address());
    n += p.print(" Dest: ");
    n += tmpdest.printTo(p);
    n += p.println("");
    return n;
}

//
// PacketTCP
//
uint16_t  PacketTCP::sourcePort() const
{
    return ntohs(_sourcePort);
}

uint16_t  PacketTCP::destPort() const
{
    return ntohs(_destPort);
}

uint32_t  PacketTCP::sequence() const
{
    return ntohl(_sequence);
}

uint32_t  PacketTCP::acknowledgement() const
{
    return ntohl(_acknowledgement);
}

uint8_t PacketTCP::headerlen() const 
{
    return (_hlen_code & 0xe0) >> 3;
}

uint8_t PacketTCP::code() const 
{
    return _hlen_code & 0x3f;
}

uint16_t  PacketTCP::window() const
{
    return ntohs(_window);
}

uint16_t  PacketTCP::checksum() const
{
    return ntohs(_checksum);
}

uint16_t  PacketTCP::urgent() const
{
    return ntohs(_urgent);
}

uint8_t* PacketTCP::payload()
{
    return (uint8_t*)this + headerlen();
}

size_t  PacketTCP::printTo(Print& p) const
{
    size_t n = 0;
    
    n += p.println("TCP");
    n += p.print(" Source Port: ");
    n += p.println(sourcePort(), DEC);
    n += p.print(" Dest Port: ");
    n += p.println(destPort(), DEC);

    n += p.print(" Sequence: ");
    n += p.println(sequence(), DEC);
    n += p.print(" Acknowledgement: ");
    n += p.println(acknowledgement(), DEC);
    n += p.print(" Window: ");
    n += p.println(window(), DEC);

    n += p.print(" Checksum: ");
    n += p.println(checksum(), HEX);

    n += p.print(" Urgent: ");
    n += p.println(urgent(), DEC);


    return n;
}

//
// PacketUDP
//

uint16_t  PacketUDP::sourcePort() const
{
    return ntohs(_sourcePort);
}

uint16_t  PacketUDP::destPort() const
{
    return ntohs(_destPort);
}

uint16_t  PacketUDP::length() const
{
    return ntohs(_length);
}

uint16_t  PacketUDP::checksum() const
{
    return ntohs(_checksum);
}

uint8_t* PacketUDP::payload()
{
    return _data;
}

uint16_t PacketUDP::payload_length()
{
    return length() - sizeof(*this);
}

size_t  PacketUDP::printTo(Print& p) const
{
    size_t n = 0;
    
    n += p.println("UDP");
    n += p.print(" Source Port: ");
    n += p.println(sourcePort(), DEC);
    n += p.print(" Dest Port: ");
    n += p.println(destPort(), DEC);
    n += p.print(" Length: ");
    n += p.println(length(), DEC);
    n += p.print(" Checksum: ");
    n += p.println(checksum(), HEX);
    return n;
}
